%Script for chromatic characterization of HTC Vive

clear all;

%Read RGB-XYZ measurement pairs for three independent channels
load MeasurementsHTCvive.mat

MpC=52;%Measurements Per Channel

%Looking for gamma value each RGB channel

for i=1:3
   
    %Normalizing RGB values from 0 to 1;    
    x = RGB((i-1)*MpC+1:i*MpC,i)./255;
    %Normalizing XYZ values from 0 to 1;
    y = XYZvalue((i-1)*MpC+1:i*MpC,2)./(XYZvalue(i*MpC,2));

    %Define characterization model
    modelfun = @(b,x)0 + 1*x(:).^b(1);
    beta0 = [1];
    
    %Apply non-linear fit
    mdl = fitnlm(x,y,modelfun,beta0);

    %Copy gamma value
    A(i,1)=mdl.Coefficients(1,1);
    B=table2array(A);
    
    %Calculate RGBprime with estimated gamma value
    RGBprime(:,i)=(RGB(:,i)./255).^(B(i,1));

    %Plot adjusted values
    plot (RGBprime((i-1)*MpC+1:i*MpC,i),y);
    plot(x,y);
    yprima=0 + 1*x(:).^B(i,1);
    hold on;
    plot(x,yprima,'.');
       
end

%Define the calibration matrix with XYZ values corresponding to max RED
%(255,0,0), max GREEN (0,255,0) y max BLUE (0,0,255).

CalibrationMatrix(1,1)=XYZvalue(MpC,1);
CalibrationMatrix(1,2)=XYZvalue(MpC*2,1);
CalibrationMatrix(1,3)=XYZvalue(MpC*3,1);

CalibrationMatrix(2,1)=XYZvalue(MpC,2);
CalibrationMatrix(2,2)=XYZvalue(MpC*2,2);
CalibrationMatrix(2,3)=XYZvalue(MpC*3,2);

CalibrationMatrix(3,1)=XYZvalue(MpC,3);
CalibrationMatrix(3,2)=XYZvalue(MpC*2,3);
CalibrationMatrix(3,3)=XYZvalue(MpC*3,3);

%Normalize the matrix with reference white luminance.
ReferenceWhite=XYZvalue(MpC*3+1,:)./XYZvalue(MpC*3+1,2);
CalibrationMatrix=CalibrationMatrix./XYZvalue(MpC*3+1,2);
CalibrationMatrixInversa=inv(CalibrationMatrix);

%Calculate XYZ values starting from RGB values using Calibration Matrix and
%gamma values
[Nrows Ncols]=size(XYZvalue);

for row=1:Nrows
    
    XYZCalculated(:,row)=((CalibrationMatrix)*((RGBprime(row,:)'))).*XYZvalue(MpC*3+1,2);
end

%Transform from XYZ to Lab for a better color difference stimation
LabCalculated=xyz2lab(XYZCalculated',XYZvalue(MpC*3+1,:));
LabMeasured=xyz2lab(XYZvalue,XYZvalue(MpC*3+1,:));

DE=ciede2000(LabMeasured(1:Nrows,:),LabCalculated(:,:));

%Calculate Average color difference for random RGB values
MeanError=mean(DE(MpC*3+2:Nrows,:))

%Calculate standard deviation
StdDesv=std(DE(MpC*3+2:Nrows,:))

          


